/**********************************************************************
 * CD Jukebox by Nathan Strong (gblues@jps.net)
 *
 * This software is covered by the GNU Public License. See COPYING
 * file for details on the license. All code contained in these files
 * is (c) 1999 Nathan Strong.
 **********************************************************************/

#include "main.h"
#include "ansi.h"

void write_led(int num, int x);
void write_tracks(int num);

struct ui_data ui;

OPTION_DATA options[] =
{
  { TRUE, do_next_track,	" >>| " },
  { FALSE, do_fast_forward,	" >>  " },
  { FALSE, do_play,		"  >  " },
  { FALSE, do_stop,		" []  " },
  { FALSE, do_pause,		" ||  " },
  { FALSE, do_rewind,		"  << " },
  { FALSE, do_previous_track,	" |<< " },
  { FALSE, NULL, "" }
};

void init_screen(void)
{
  ui.playlist  = generate_playlist(FALSE);
  if( ui.playlist == NULL )
  {
    fprintf(stderr, "Error: no music discs found.\n");
    exit_cdplayer();
  }
  ui.cur_cd      = ui.playlist->drive;
  ui.cur_track   = ui.playlist;
  ui.cur_min     = 0;
  ui.cur_sec     = 0;
  ui.time_format = COUNT_UP_TRACK;
  ui.options     = 0;
  ui.num_tracks  = ui.cur_cd->toc->cdth_trk1;
  ui.commands    = options;

  CLEAR_SCREEN;
  MOVE_CURSOR(0,0);
  printf(" _________________________________________________________________\n");
  printf("|%s%s|\n", "                             ",
         "                                    ");
  printf("|%s%s|\n", "                             ",
         "                                    ");
  printf("|%s%s|\n", "                             ",
         "                                    ");
  printf("|_________________________________________________________________|\n");
}

void update_tracks(void)
{
  int msb, lsb = ui.cur_track ? ui.cur_track->track : 0;
  msb = lsb / 10;
  lsb %= 10;
  write_led(msb, 3);
  write_led(lsb, 6);
  write_tracks(ui.num_tracks);
  MOVE_CURSOR(0, 6);
  printf("Current drive: %s", ui.cur_cd->filename);
}

void write_tracks(int num)
{
  int i, j, count = 0;
  int msb, lsb;
  static char emptystr[] = "::::";
  char str[5];
  memcpy(str, emptystr, 5);
  for( i = 0; i < 3; i++)
  {
    MOVE_CURSOR(31, 2+i);
    for( j = 0; j < 7; j++ )
    {
      if( count < num )
      {
        count++;
        msb = count / 10;
        msb = msb > 9 ? '9' : '0'+ msb;
        lsb = count % 10;
        lsb += '0';
        str[1] = msb ? msb : ':';
        str[2] = lsb;
        printf("%s%s\033[0m ", BR_CYAN, str);
      }
      else
        printf("%s%s\033[0m ", BR_BLACK, emptystr);
    }
  }
}

void write_led(int num, int x)
{
  static char led_zero[7];
  static char led[7];
  memcpy(led, led_zero, 7);
  switch(num)
  {
    default:
      break;
    case 0:
      led[0] = 1;
      led[1] = 1;
      led[3] = 1;
      led[4] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
    case 1:
      led[3] = 1;
      led[6] = 1;
      break;
    case 2:
      led[0] = 1;
      led[2] = 1;
      led[3] = 1;
      led[4] = 1;
      led[5] = 1;
      break;
    case 3:
      led[0] = 1;
      led[2] = 1;
      led[3] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
    case 4:
      led[1] = 1;
      led[2] = 1;
      led[3] = 1;
      led[6] = 1;
      break;
    case 5:
      led[0] = 1;
      led[1] = 1;
      led[2] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
    case 6:
      led[0] = 1;
      led[1] = 1;
      led[2] = 1;
      led[4] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
    case 7:
      led[0] = 1;
      led[3] = 1;
      led[6] = 1;
      break;
    case 8:
      led[0] = 1;
      led[1] = 1;
      led[2] = 1;
      led[3] = 1;
      led[4] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
    case 9:
      led[0] = 1;
      led[1] = 1;
      led[2] = 1;
      led[3] = 1;
      led[5] = 1;
      led[6] = 1;
      break;
  }
  MOVE_CURSOR(x, 2);
  printf(" %s_\033[0m ", led[0] ? BR_CYAN : BR_BLACK);
  MOVE_CURSOR(x, 3);
  printf("%s|\033[0m%s_\033[0m%s|\033[0m",
         led[1] ? BR_CYAN : BR_BLACK,
         led[2] ? BR_CYAN : BR_BLACK,
         led[3] ? BR_CYAN : BR_BLACK);
  MOVE_CURSOR(x, 4);
  printf("%s|\033[0m%s_\033[0m%s|\033[0m",
         led[4] ? BR_CYAN : BR_BLACK,
         led[5] ? BR_CYAN : BR_BLACK,
         led[6] ? BR_CYAN : BR_BLACK);
}

void update_options(void)
{
  int i;

  MOVE_CURSOR(15, 8);
  for( i = 0; ui.commands[i].name[0] != '\0'; i++ )
    printf("%s%s\033[0m",
           ui.commands[i].selected ? REVERSE : "",
           ui.commands[i].name);
  MOVE_CURSOR(0, 9);
}

void boot_ui(void)
{
  init_screen();
  SET_BIT(ui.dirty, DIRTY_UI | DIRTY_CLOCK | DIRTY_TRACK);
  update_ui();
  program_loop();
}

void update_timer(void)
{
  MOVE_CURSOR(20, 3);
  printf("%s.\033[0m", BR_CYAN);
  MOVE_CURSOR(20, 4);
  printf("%s.\033[0m", BR_CYAN);
  if( !ui.playing )
  {
    write_led(10, 14);
    write_led(10, 17);
    write_led(10, 21);
    write_led(10, 24);
  }
  else
  {
    int msb, lsb;
    msb = ui.cur_min / 10;
    lsb = ui.cur_min % 10;
    write_led(msb, 14);
    write_led(lsb, 17);
    msb = ui.cur_sec / 10;
    lsb = ui.cur_sec % 10;
    write_led(msb, 21);
    write_led(lsb, 24);
  }
}
  
void update_ui(void)
{
  if( IS_SET(ui.dirty, DIRTY_TRACK) )
    update_tracks();
  if( IS_SET(ui.dirty, DIRTY_CLOCK) )
    update_timer();
  if( IS_SET(ui.dirty, DIRTY_UI) )
    update_options();
  MOVE_CURSOR(0, 9);
  ui.dirty = 0;
  fflush(NULL);
}
